
// Thermomtre avec temprature intrieure et extrieure.
// Quartz  3.2768Mhz
// prdiviseur  16, incrmentation de RTCC toute les 19.53us dbordement toute les 5ms
// Affichage sur un LCD 1 ligne.
// Traitement de l'inter toute les 5ms
// traitement de la temprature intrieure toute les 12.8s
// Reception de la temprature extrieure par interruption
 
#include "16f84.h"
#include "int16CXX.H"



//---------------------------Constantes----------------------------------

#define	cid	0b00000110;	// adresse de la carte capteur externe

#define	clong	10		// constante  12.8s
#define	ctrtim	41		// fin timer transmission front montant il y a 800us
#define	ctrerr	87		// fin timer erreur front montant il y a 1700us
#define	cinatt	60		// attente avant nouvelle scrutation clavier 300ms

//---------------------------Variables globales---------------------------------------

// Temprature interieure
char 	titim;		// timer pour commencer la rcupration
char	tisig;		// signe - ou rien
char 	ti1;		// valeur de la temprature intrieure
char	timax;		// temprature intrieure max
char	timin;		// temprature intrieure min
char	tisimax;	// signe de la temprature max
char	tisimin;	// Signe de la temprature min

// Temprature extrieure
char 	te1;		// valeur de la temp extrieure
char	tesig;		// signe - ou rien
char	temax;		// temprtature extrieure max
char	temin;		// tempraturez extrieure min
char	tesimax;	// signe de la temprature max
char	tesimin;	// Signe de la temprature min

// transmission radio

// Affichage et timer
char	timinter;	// timer de scrutation de l'interrupteur
char    timminmax;	// determination de lappui long sur l'interrupteur minmax
char	tim5ms;		// compteur des 5ms, dbordement toutes les 1.28s
char	tim128;		// compteur des 1.28s
char	timaf;		// timer d'affichage
char	afeta;		// etat de l'affichage
                              	// 7
				// 6
				// 5
				// 4  affichage valide
				// 3  affichage temp ext=1  int=0
				// 2  affichage du minimum
				// 1  affichage du maximum
				// 0  affichage du present
char	tempsig;	// signe  afficher
char	tempdix;	// diximes  afficher
char	tempuni;	// unites  afficher
char	tempdiz;	// dizaines  afficher

//------------------------I/O-------------------------------------------

bit	intext @ PORTA.0;	// interrupteur intrieur exterieur
bit	dq @ PORTA.1;		// temprature intrieure I/O
bit	clk @ PORTA.2;		// horloge temprature interieure
bit 	rst @ PORTA.3;		// temprature interieure reset
bit	minmax @ PORTA.4;	// interrupteur min, max, mormal
bit	te @ PORTB.0;		// temprature exterieure
bit	aft @ PORTB.1;		// affichage sous tension
bit	LCD_RS @ PORTB.2;	// register select de l'afficheur
bit	LCD_EN @ PORTB.3;	// enable de l'afficheur
bit	LCD_D4 @ PORTB.4;
bit	LCD_D5 @ PORTB.5;



//-------------------------equivalence-----------------------------------

#define	sortie	TRISA=0b00010001;	// sens des broches porta avec dq en sortie
#define	entree	TRISA=0b00010011;	// sens des broches porta avec dq en entree

//--------------------------------interruption----------------------------

#pragma origin 4



interrupt interruption(void)
{
int_save_registers

  if (T0IF)			// interruption timer
  {
  tim5ms++;			// incrementation timer 5ms
  timinter++;			// incrementation du timer pour la gestion de l'interrupteur
  if (tim5ms==0)		// 1.28 seconde atteint
    { 

	titim++;				// incrementation timer de rcupration de t interieure
	tim128++;				// incrementation timer 1.28s
	timaf++;				// incrementation timer d'affichage
	if (timaf==clong)			// eteindre l'affichage au bout de clong secondes
		{
		aft=1;
		afeta.4=0;
		}
    }
  T0IF = 0; 
  }

  if (INTF)			// interruption RB0
  {
  INTF = 0;
  }

int_restore_registers
}

//-------------------initialisation-------------------------------------

void initialisation()
{
OPTION=0b11010011;	// registre option    prediv=16
PORTA=0b00010100;	// porta  l'initialisation
PORTB=0b00000010;	// portb  l'initialisation
sortie
TRISB=0b00000001;	// sens des broches portb
INTCON=0b10110000;	// autorisation des interruptions
tim5ms=0;
tim128=0;
afeta=0b00010001;	// initialisation de l'affichage
timaf=0;
titim=30;		// initialisation des tempratures
tisimax='-';
timax=0x92;
tisimin=' ';
timin=0xFA;
tesig=' ';
te1=0xFA;
tesimax='-';
temax=0x92;
tesimin=' ';
temin=0xFA;

}

//-------------------------delay en 5ms-----------------------------------

void delay_5ms(char c)
{
char fin;
fin = c + tim5ms;
do clrwdt(); while(tim5ms != fin);
}
//-------------------------delay en 20us-----------------------------------

void delay_10us(char c)
{
char cpt;
cpt = 0 ;
do
 {
  nop();
  nop();
  nop();
  clrwdt();
  cpt++;
 }
while(cpt != c);
}

//--------------------------------crire en code rduit---------------------------------
//	LCD_D4 = PORTB.4    LCD_D5 = PORTB.5  etc

void lcd_write_fast(unsigned char c)
{

	PORTB &= 0b00001111;	// RB4  RB7 = 0
	W = c & 0b11110000;
	PORTB |= W; 		//quartet haut de c sur RB4  RB7
	LCD_EN = 1;
	LCD_EN = 0;
	PORTB &= 0b00001111;	// RB4  RB7 = 0
	c = swap (c);
	W = c & 0b11110000;
	PORTB |= W; 		//quartet haut de c sur RB4  RB7
	LCD_EN = 1;
	LCD_EN = 0;
	delay_10us(8);
}

//-------------------------------Clear and home the LCD---------------------------------

void lcd_clear(void)
{
	LCD_RS = 0;
        lcd_write_fast(0x1);
	delay_5ms(1);
}

//----------------------write a string of chars to the LCD-------------------------------

void lcd_puts(const char * s)
{
	LCD_RS = 1;		// write characters
	while(*s) lcd_write_fast(*s++);
}

//---------------------------write one character to the LCD--------------------------------

void lcd_putch(unsigned char c)
{
	LCD_RS = 1;		// write characters
	lcd_write_fast(c);
}

//---------------------------Go to the specified position----------------------------------

void lcd_goto(unsigned char pos)
{
	LCD_RS = 0;
	lcd_write_fast(0x80 + pos);
}


//-------------------------Go to specific position to program caracter---------------------

void lcd_goto_cgram(unsigned char pos)
{
	LCD_RS = 0;
	lcd_write_fast(0x40 + pos);
}


//--------------------initialise the LCD - put into 4 bit mode-------------------------

void lcd_init(void)
{
	LCD_RS = 0;		// write control bytes
	delay_5ms(5);		// power on delay
	LCD_D4 = 1;		// init!
	LCD_D5 = 1;
	LCD_EN = 1;
	LCD_EN = 0;	
	delay_5ms(3);
	LCD_EN = 1;		// init!
	LCD_EN = 0;	
	delay_10us(14);
	LCD_EN = 1;		// init!
	LCD_EN = 0;	
	delay_5ms(3);
	LCD_D4 = 0;		// set 4 bit mode
	LCD_EN = 1;
	LCD_EN = 0;	
	delay_10us(8);
	lcd_write_fast(0x28);	// 4 bit mode, 1/16 duty, 5x8 font, 2 lines
	lcd_write_fast(0x0C);	// display on
	lcd_write_fast(0x06);	// entry mode advance cursor
	lcd_write_fast(0x01);	// clear display and reset cursor
	delay_5ms(1);
}

//--------------------criture de 8 bit sur le ds1620--------------------

void temp_write_8bits(unsigned char nb)
{
sortie
clk=0;				// criture de nb dans ds1620
dq=nb.0;
clk=1;
clk=0;
dq=nb.1;
clk=1;
clk=0;
dq=nb.2;
clk=1;
clk=0;
dq=nb.3;
clk=1;
clk=0;
dq=nb.4;
clk=1;
clk=0;
dq=nb.5;
clk=1;
clk=0;
dq=nb.6;
clk=1;
clk=0;
dq=nb.7;
clk=1;
}

//--------------------lecture de 8 bits sur le ds1620---------------------

void temp_read_8bits(void)
{

entree
clk=0;
ti1.0=dq;
clk=1;
clk=0;
ti1.1=dq;
clk=1;
clk=0;
ti1.2=dq;
clk=1;
clk=0;
ti1.3=dq;
clk=1;
clk=0;
ti1.4=dq;
clk=1;
clk=0;
ti1.5=dq;
clk=1;
clk=0;
ti1.6=dq;
clk=1;
clk=0;
ti1.7=dq;
clk=1;
}

//-------------------------ecriture de la config dans le ds1620-------------------------
void temp_cfg_write(char nb)		// le temps d'criture prend 10 ms
{
rst=1;
temp_write_8bits(0x0C);
temp_write_8bits(nb);
rst=0;
}

//-------------------------start conversion du ds1620-------------------------
void temp_start(void)
{
rst=1;
temp_write_8bits(0xEE);
rst=0;
}


//-------------------------lecture de la temprature du ds1620 et traitement du max et min-------------
void temp_read(void)

{
rst=1;				//recuperation dans ti1 de la temprature
temp_write_8bits(0xAA);
temp_read_8bits();
clk=0;				//reception du signe de la temprature et traitement du min et max
if(dq==0)
   {
	clk=1;		// temperature>0
	rst=0;
	tisig=' ';
	if (tisimax=='-')
	  {
	   tisimax=' ';
	   timax=ti1;
	  }
	else if (ti1>timax) timax=ti1;
	if (tisimin==' ')
	  {
	   if (ti1<timin) timin=ti1;
	  }
   }
else
   {
	clk=1; 		// temperature<0
	rst=0;
	tisig='-';
	if (tisimin==' ')
	  {
	   tisimin='-';
	   timin=ti1;
	  }
	else if (ti1<timin) timin=ti1;
	if (tisimax=='-')
	  {
	   if (ti1>timax) timax=ti1;
	  }
   }

}
//---------------------------conversion de la temprature en dizaine unite dixime et affichage-------------

void affichage(char valeur, char signe)
{
char cpt;

     if(valeur.0 ==0) tempdix='0'; else tempdix='5';	// ecriture des diximes.

     if (signe=='-')			// si la temprature est < 0
       {
	tempsig='-';
	tempdiz=0X35;		//0x35 = 5 en alphanumerique
	tempuni=0X35;	
	cpt=0X92;		//-55C = 92
	while (cpt < valeur) 	//decrementation de tempuni et tempdiz pour arriver  l'affichage correct
	{
	  cpt++;
	  cpt++;
	  tempuni--;
	  if (tempuni==0X2F)
	  {
		tempuni=0X39;
		tempdiz--;
	  } 
	  if (cpt==0) cpt=0xff;		// correction pour le cas ou cpt est pass  0
	}
	if (valeur<0x96)		// correction dans le cas ou temp < -50  (affichage de -)
		{
		tempdix='-';
		tempuni='-';
		tempdiz='-';
		tempsig=' ';
		}
       }
     else 				// si la temprature > 0 alors 
	{
	 tempsig=' ';
	 tempdiz=0x30;			//0x30 = 0 en alphanumerique
	 tempuni=0X30;
	 cpt=1;				// 0,5c = 1
	 while (cpt < valeur)		//incrementation de tempuni et tempdiz pour arriver a l'affichage correct
	  {
	   cpt++;
	   cpt++;
	   tempuni++;
	   if (tempuni==0X3A)
	    {
		 tempuni=0X30;
		 tempdiz++;
	    }
	   if (valeur>=0xC7)		// correction dans le cas ou temp > 100  (affichage de -)
		{
		cpt=valeur;
		tempdix='-';
		tempuni='-';
		tempdiz='-';
		tempsig=' ';
		}
	  }
        }
lcd_goto(2);
lcd_putch(tempsig);
lcd_putch(tempdiz);
lcd_putch(tempuni);
lcd_goto(6);
lcd_putch(tempdix);
}
//------------------cration de caractres------------------------------
void lcd_create(void)
{
lcd_goto_cgram(0x08);
lcd_puts("\x04\x0A\x11\x1F\x11\x15\x1F");	// une maison en 1
lcd_putch(0);
lcd_puts("\x0E\x1F\x1F\x0E\x14\x0D\x06");	// une fleur en 2
lcd_putch(0);
lcd_puts("\x18\x18\x03\x04\x04\x04\x03");	// C en 3
lcd_putch(0);
lcd_puts("\x08\x1c\x08");			// Valeur max en 4
lcd_putch(0);
lcd_putch(0);
lcd_putch(0);
lcd_putch(0);
lcd_putch(0);
lcd_putch(0);					// Valeur min en 5
lcd_putch(0);
lcd_putch(0);
lcd_putch(0);
lcd_putch(0);
lcd_putch(0x1c);
lcd_putch(0);
lcd_putch(0);
}

//----------------- fonction principale----------------------------------

void main()
{
				
initialisation();
temp_cfg_write(0b00000011);
aft=0;				//affichage  l'initialisation
lcd_init();
lcd_clear();
lcd_create();

lcd_goto(0);			//affichage de la temprature interieure
lcd_puts("\x01  --,-\x03");

while (1) 
{
clrwdt();

//if (aft==1) sleep();		// si il n'y a pas d'affichage mettre le micro en sleep 

if (titim==30)	temp_start();	// mise en fonction de la conversion de la temprature interieure

if (titim==31)			// rcupration de la temprature interieure et affichage si necessaire
    {
    temp_read();
    if (afeta==0b00010001) affichage(ti1,tisig);
    titim=0;
    }
if (intext==1)			// si appui sur l'interrupteur et 300ms de pass 
    {
     if (timinter >= cinatt)
	{
         timinter = 0 ;
	 if (afeta.4==0)	// si affichage invalide alors affichage
	    {
	      aft=0;
	      lcd_init();
	      lcd_clear();
	      lcd_create();
	      lcd_goto(0);
	      lcd_puts("\x01    , \x03");
	      affichage(ti1,tisig);
	      timaf=0;
	      afeta=0b00010001;	
            }
	 else
	    {
	      if (afeta.3==0)	//si affichage temprature interieure alors affichage temp exterieure 
	       {
		lcd_goto(0);
		lcd_puts("\x02    , \x03");
		affichage(te1,tesig);
	        timaf=0;
		afeta=0b00011001;
	       }
	      else		//si affichage temprature extrieure alors affichage temp int 
	       {
		lcd_goto(0);
		lcd_puts("\x01    , \x03");
		affichage(ti1,tisig);
		timaf=0;
		afeta=0b00010001;
	       }
	   }
	}
    }

if (minmax==0) timminmax=0;

if (minmax==1)
    {
     if (timinter >= cinatt)
	{
	 timminmax += 1;
         timinter = 0 ;
	 if (timminmax >= 5)	// le doigt est rest sur minmax donc effacement des min et max.
	  {
	    timin=ti1;
	    timax=ti1;
	    tisimin=tisig;
	    tisimax=tisig;
	    temin=te1;
	    temax=te1;
	    tesimin=tesig;
	    tesimax=tesig;
	  }
	if (afeta.1==1)		// si affichage max
	    {
		if (afeta.3==0)		// si temp interieure alors affichage min interieur
		{
	          lcd_goto(0);
		  lcd_puts("\x01\x05   , \x03");
		  affichage(timin,tisimin);
	          timaf=0;
		  afeta.1=0;
		  afeta.2=1;
		}
		if (afeta.3==1)		// si temp exterieure alors affichage min exterieur
		{
	          lcd_goto(0);
		  lcd_puts("\x02\x05   , \x03");
		  affichage(temin,tesimin);
	          timaf=0;
		  afeta.1=0;
		  afeta.2=1;
		}
	    }

	 else
	   {
	    if (afeta.0==1)		// si affichage present
	    {
		if (afeta.3==0)		// si temp interieure alors affichage max interieur
		{
	          lcd_goto(0);
		  lcd_puts("\x01\x04   , \x03");
		  affichage(timax,tisimax);
	          timaf=0;
		  afeta.0=0;
		  afeta.1=1;
		}
		if (afeta.3==1)		// si temp exterieure alors affichage max exterieur
		{
	          lcd_goto(0);
		  lcd_puts("\x02\x04   , \x03");
		  affichage(temax,tesimax);
	          timaf=0;
		  afeta.0=0;
		  afeta.1=1;
		}
	    }
	    else			// si affichage min
	    {
		if (afeta.3==0)		// si temp interieure alors affichage present interieur
		{
	          lcd_goto(0);
		  lcd_puts("\x01    , \x03");
		  affichage(ti1,tisig);
	          timaf=0;
		  afeta.2=0;
		  afeta.0=1;
		}
		if (afeta.3==1)		// si temp exterieure alors affichage present exterieur
		{
	          lcd_goto(0);
		  lcd_puts("\x02    , \x03");
		  affichage(te1,tesig);
	          timaf=0;
		  afeta.2=0;
		  afeta.0=1;
		}
	    }
	   }
	}	

    }
}
}

